' Layout_UpdateOnClose_Handler:
' Revision: v1.0.2
' (c) 2016 steje
'
' This is a script for written for Directory Opus v12.
' See http://www.gpsoft.com.au/redirect.asp?page=scripts for development information.

' =====================================================================================
' =========================   EVENT HANDLER: OnCloseLister   ==========================
'
' The purpose of this script is to provide a way for users to automatically update a
' saved lister layout when closing a lister that was launched from a layout. By default
' the script ignores the 'default' lister layout, since Opus has it's own setting in
' Prefs for handling that special layout. The script provides options to override both
' the handling of the default lister as well as to allow user control over which saved
' layouts are or are not updated on close:
'
' ExcludeLayoutList:     Set this option to define the names of the desired saved layout
'                        the script should NOT update on lister close
' IncludeDefaultLayout:  Set this option to True in order to override the settings in
'                        Prefs (if disabled) to automatically update the default lister
'                        on close
'
' The scripts behavior can be enabled/disabled by either of the following settings:
'
' Script configuration:          The scripts listing under Preferences
' Script configuration option:   Setting the Always_On option to True/False
' An Opus environment variable:  layout_updateonclose
'
' The first setting is controlled by checking or unchecking the scripts listing in Prefs
' The second setting is controlled via the Always_On option under the scripts listing in Prefs
' The third setting can be controlled via any standard button or hotkey in Opus that
' toggles an environment variable of either global or lister scope, e.g.:
'
'                        @toggle:if $lst:layout_updateonclose
'
'                        @ifset:$lst:layout_updateonclose
'                        @set lst:layout_updateonclose
'
'                        @ifset:else
'                        @set lst:layout_updateonclose=true
'
' =====================================================================================

Option Explicit
Dim SCRIPT_DEBUG
SCRIPT_DEBUG = False


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Called by Directory Opus to initialize the script
Function OnInit(ByRef initData)

   DOpus.Output("Initializing...")

   ' Provide basic information about the Script
   initData.min_version = "12.2"
   initData.name = "Layout_UpdateOnClose_Handler"
   initData.desc = "Event Handler for the OnCloseLister event. Used to update a user defined" &_
                   "lister Layout when one of it's listers are closed."
   initData.copyright = "(c) 2016 steje"
   initData.version = "v1.0.2"
   initData.default_enable = True

   '////////////////////////////////////////////////
   ' The following options are user configurable via the Configure button under the scripts listing in Prefs
   '////////////////////////////////////////////////
   ' Set Always_On to True in order to enable script execution without the use of environment variables
   initData.config.Always_On = True
   '////////////////////////////////////////////////
   ' Set DEBUG flag to True in order to enable logging messages to the Opus Output Window
   initData.config.DEBUG = False
   '////////////////////////////////////////////////
   ' Set DEBUG_CLEAR flag to True in order to clear log messages from the Opus Output Window between script runs
   initData.config.DEBUG_CLEAR = False
   '////////////////////////////////////////////////
   ' Set ExcludeLayoutList to define the names of the desired saved layouts the script should NOT update on lister close
   initData.config.ExcludeLayoutList = DOpus.NewVector("")
   '////////////////////////////////////////////////
   ' Set IncludeDefaultLayout to True to override the settings in Prefs (if disabled) to automatically update the default lister on close
   initData.config.IncludeDefaultLayout = False
   '////////////////////////////////////////////////
   ' Set LayoutSaveOptions to additional options support by the 'Prefs LAYOUTSAVE' command. If specifying actual LAYOUTSAVE option args, make them first followed by a comma
   initData.config.LayoutSaveOptions = ""
   '////////////////////////////////////////////////
   ' Set LayoutSaveSingle to True in order to save only the single lister being closed in the updated lister layout
   initData.config.LayoutSaveSingle = True
   '////////////////////////////////////////////////
   ' Set SetAsDefaultForce to True in order to suppress the confirmation and success prompts when setting the Default Lister manually
   initData.config.SetAsDefaultForce = False
   '////////////////////////////////////////////////
   ' Set SetAsDefaultQuiet to True in order to suppress the success prompt when setting the Default Lister
   initData.config.SetAsDefaultQuiet = False
   '////////////////////////////////////////////////
   ' Set QualKey to the qualifier key that you want to ENABLE the scripts execution
   initData.config.QualKey = DOpus.NewVector(0, "none", "shift", "ctrl", "alt", "lwin", "rwin")
   '////////////////////////////////////////////////
   ' Set QualKey_Disable to the qualifier key that you want to DISABLE the scripts execution
   initData.config.QualKey_Disable = DOpus.NewVector(0, "none", "shift", "ctrl", "alt", "lwin", "rwin")
   '////////////////////////////////////////////////
   ' Set descriptions for all script options
   initData.config_desc = DOpus.NewMap("Always_On", "Set this option to True in order to enable script execution without the use of environment variables.", _
                                       "DEBUG", "Set this option to True to enable logging to the Opus script log.", _
                                       "DEBUG_CLEAR", "Set this option to True clear messages logged to the Opus script log between script runs.", _
                                       "ExcludeLayoutList", "Set this option to define the names of the desired saved layout the script should NOT update on lister close.", _
                                       "IncludeDefaultLayout", "Set this option to True in order to override the settings in Prefs (if disabled) to automatically update the default lister on close.", _
                                       "LayoutSaveOptions", "Set this to define additional options for the 'Prefs LAYOUTSAVE' command.", _
                                       "LayoutSaveSingle", "Set this option to True in order to save only the single lister being closed in the updated lister layout.", _
                                       "SetAsDefaultForce", "Set this option to True in order to suppress the confirmation and success prompts when setting the Default Lister manually.", _
                                       "SetAsDefaultQuiet", "Set this option to True in order to suppress the success prompt when setting the Default Lister.", _
                                       "QualKey", "Set this option to the qualifier key that you want to ENABLE the scripts execution.", _
                                       "QualKey_Disable", "Set this option to the qualifier key that you want to DISABLE the scripts execution.")

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Called after a lister has been closed
Function OnCloseLister(ByRef closeListerData)
   Dim objCmd

   Set objCmd = DOpus.NewCommand
   If (objCmd.IsSet("$glob:SCRIPT_DEBUG")) Then SCRIPT_DEBUG = True
   If ((Script.config.DEBUG) Or (SCRIPT_DEBUG)) And (Script.config.DEBUG_CLEAR) Then DOpus.ClearOutput
   'logMsg("OnCloseLister()")

   logMsg("")
   logMsg("----------------------------------------------------------------")
   logMsg("OnCloseLister:  " & closeListerData.lister.layout & "->" & closeListerData.lister.title)
   logMsg("----------------------------------------------------------------")
   If (executeScript(closeListerData)) Then
      'Check if this is the default lister layout and the override option is not set
      If ((closeListerData.lister.layout = "") And (Not (Script.config.IncludeDefaultLayout))) Then
         logMsg("The Default lister layout was used, exit...")
      Else
         Call Layout_Update(closeListerData.lister)
      End If
   End If

   Set objCmd = Nothing

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Main worker function of the script for OnCloseLister events
Function Layout_Update(ByRef objSrcLister)
   Dim fCommandArgs, fExcludeLayoutListMatch
   Dim objCmd, objLister
   Dim strLayout, strCommand, strCommandArgs
   'logMsg("Layout_Update()")

   Layout_Update = False
   fCommandArgs = False
   fExcludeLayoutListMatch = False
   Set objLister = objSrcLister
   strLayout = objLister.layout

   fExcludeLayoutListMatch = getExcludeLayoutListMatch(strLayout)
   If (fExcludeLayoutListMatch) Then
      logMsg("Found [" & strLayout & "] in ExcludeLayoutList, exit...")
   Else
      logMsg("Updating layout:  " & strLayout)

      Set objCmd = DOpus.NewCommand
      objCmd.SetSourceTab(objLister.activetab)
      strCommand = ""
      strCommandArgs = ""

      'If there is no layout name it's the default lister so run setdefaultlister
      If (strLayout = "") Then
         'Get the setasdefault options from the script config options
         If (Script.config.SetAsDefaultForce) Then
            strCommandArgs = "=force"
            fCommandArgs = True
         End If
         If (Script.config.SetAsDefaultQuiet) Then
            If (fCommandArgs) Then
               strCommandArgs = strCommandArgs & ","
            Else
               strCommandArgs = "="
            End If
            strCommandArgs = strCommandArgs & "quiet"
         End If
         strCommand = "Prefs SETDEFAULTLISTER" & strCommandArgs
      Else
         'Get the layoutsave options from the script config options
         If (Script.config.LayoutSaveSingle) Then
            strCommandArgs = ",single"
         End If
         strCommand = "Prefs LAYOUTSAVE=updatecurrent" & strCommandArgs
      End If

      'logMsg("  Run:  " & strCommand)
      objCmd.RunCommand(strCommand)
   End If

   Set objLister = Nothing
   Set objCmd = Nothing

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Check for various things that determine if the script should run or not
Function executeScript(ByRef listerCloseData)
   Dim QUALKEY
   Dim objCmd
   'logMsg("executeScript()")

   executeScript = False

   ' Check for any of the script options or environment variables that enable the scripts execution
   Set objCmd = DOpus.NewCommand
   If (Script.config.Always_On) Then
      'logMsg("Script is set to 'always on'")
   ElseIf(objCmd.IsSet("$glob:layout_updateonclose")) Then
      'logMsg("[layout_updateonclose] is set: [global]")
   ElseIf (objCmd.IsSet("$lst:layout_updateonclose")) Then
      'logMsg("[layout_updateonclose] is set: [lister]")
   Else
      logMsg("Not enabled, exit...")
      Set objCmd = Nothing
      Exit Function
   End If

   ' Check for any qualifier key that might have been used and recast as int we can test against vector values
   If (listerCloseData.qualifiers = "none") Then
      QUALKEY = 0
   ElseIf (listerCloseData.qualifiers = "shift") Then
      QUALKEY = 1
   ElseIf (listerCloseData.qualifiers = "ctrl") Then
      QUALKEY = 2
   ElseIf (listerCloseData.qualifiers = "alt") Then
      QUALKEY = 3
   ElseIf (listerCloseData.qualifiers = "lwin") Then
      QUALKEY = 4
   ElseIf (listerCloseData.qualifiers = "rwin") Then
      QUALKEY = 5
   End If

   ' Check for DISABLING qualifier key settings
   If ((QUALKEY = Script.config.QualKey_Disable) And (QUALKEY > 0)) Then
      logMsg("The qualifier key to DISABLE was used, exit...")
      Set objCmd = Nothing
      Exit Function
   End If

   ' Check for ENABLING qualifier key settings
   If Not (QUALKEY = Script.config.QualKey) Then
      logMsg("No valid qualifier key was used, exit...")
      Set objCmd = Nothing
      Exit Function
   End If

   executeScript = True
   Set objCmd = Nothing

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Check if the current layout name is in the exclude list in the script config options
Function getExcludeLayoutListMatch(ByRef strLayout)
   Dim i
   Dim strExcludeLayoutListEntry
   'logMsg("getExcludeLayoutListMatch()")

   getExcludeLayoutListMatch = False

   ' Check if the ExcludeLayoutList is empty and if so - nothing to do, exit...
   If (Script.config.ExcludeLayoutList.count = 1) And (Script.config.ExcludeLayoutList(0) = "") Then
      logMsg("ExcludeLayoutList is empty, exit...")
      Exit Function
   End If

   ' Parse the ExcludeLayoutList and determine if the current layout is listed
   i = 0
   logMsg("Parsing ExcludeLayoutList...")
   For Each strExcludeLayoutListEntry in (Script.config.ExcludeLayoutList)
      logMsg("  ExcludeLayoutList(" & i & "):  " & strExcludeLayoutListEntry)

      If ((StrComp(LCase(strLayout), LCase(strExcludeLayoutListEntry), vbTextCompare)) = 0) Then
         getExcludeLayoutListMatch = True
         Exit Function
      End If

      i = i + 1
   Next

End Function


'//////////////////////////////////////////////////////////////////////////////////////////////////
' Subroutine that uses DEBUG variables to control whether logging is performed or not
Sub logMsg(ByRef message)

   If ((Script.config.DEBUG) Or (SCRIPT_DEBUG)) Then DOpus.Output(message)

End Sub
